@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -164,7 +164,7 @@ int get_arg_equation(arg_struct *arg, vartype_equation **eq) {
         v = recall_var(arg->val.text, arg->length);
         if (v == NULL)
             return ERR_NONEXISTENT;
-    } else if (arg->type == ARGTYPE_STK) {
+    } else { // arg->type == ARGTYPE_STK
         int idx;
         switch (arg->val.stk) {
             case 'X': idx = 0; break;
@@ -185,6 +185,19 @@ int get_arg_equation(arg_struct *arg, vartype_equation **eq) {
     return ERR_NONE;
 }
 
+bool dim_to_int4(vartype *dim, int4 *res) {
+    phloat d = ((vartype_real *) dim)->x;
+    if (d <= -2147483648.0 || d >= 2147483648.0)
+        return false;
+    int4 dd = to_int4(d);
+    if (dd == 0)
+        return false;
+    if (dd < 0)
+        dd = -dd;
+    *res = dd - 1;
+    return true;
+}
+
 int recall_result_silently(vartype *v) {
     if (flags.f.stack_lift_disable) {
         if (sp == -1)
@@ -513,6 +526,8 @@ bool vartype_equals(const vartype *v1, const vartype *v2) {
         case TYPE_REALMATRIX: {
             const vartype_realmatrix *x = (const vartype_realmatrix *) v1;
             const vartype_realmatrix *y = (const vartype_realmatrix *) v2;
+            if (x->array == y->array)
+                return true;
             int4 sz, i;
             if (x->rows != y->rows || x->columns != y->columns)
                 return false;
@@ -539,6 +554,8 @@ bool vartype_equals(const vartype *v1, const vartype *v2) {
         case TYPE_COMPLEXMATRIX: {
             const vartype_complexmatrix *x = (const vartype_complexmatrix *) v1;
             const vartype_complexmatrix *y = (const vartype_complexmatrix *) v2;
+            if (x->array == y->array)
+                return true;
             int4 sz, i;
             if (x->rows != y->rows || x->columns != y->columns)
                 return false;
@@ -556,6 +573,8 @@ bool vartype_equals(const vartype *v1, const vartype *v2) {
         case TYPE_LIST: {
             const vartype_list *x = (const vartype_list *) v1;
             const vartype_list *y = (const vartype_list *) v2;
+            if (x->array == y->array)
+                return true;
             if (x->size != y->size)
                 return false;
             int4 sz = x->size;
@@ -638,7 +657,10 @@ int generic_comparison(const vartype *x, const vartype *y, char which) {
 
 int anum(const char *text, int len, phloat *res) {
     char buf[50];
+    int src_pos = 0;
+    retry:
     bool have_mant = false;
+    bool have_mant_digits = false;
     bool neg_mant = false;
     bool have_radix = false;
     bool have_exp = false;
@@ -646,7 +668,7 @@ int anum(const char *text, int len, phloat *res) {
     int exp_pos = 0;
     int buf_pos = 0;
     buf[buf_pos++] = '+';
-    for (int src_pos = 0; src_pos < len; src_pos++) {
+    for (; src_pos < len; src_pos++) {
         char c = text[src_pos];
         if (!flags.f.decimal_point)
             if (c == '.')
@@ -661,6 +683,7 @@ int anum(const char *text, int len, phloat *res) {
             } else if (c >= '0' && c <= '9') {
                 buf[buf_pos++] = c;
                 have_mant = true;
+                have_mant_digits = true;
             } else if (c == '.') {
                 buf[buf_pos++] = '0';
                 buf[buf_pos++] = '.';
@@ -674,12 +697,15 @@ int anum(const char *text, int len, phloat *res) {
                 neg_mant = !neg_mant;
             } else if (c >= '0' && c <= '9') {
                 buf[buf_pos++] = c;
+                have_mant_digits = true;
             } else if (c == '.') {
                 if (!have_radix) {
                     buf[buf_pos++] = c;
                     have_radix = true;
                 }
             } else if (c == 'E' || c == 'e' || c == 24) {
+                if (!have_mant_digits)
+                    goto retry;
                 buf[buf_pos++] = 'e';
                 exp_pos = buf_pos;
                 buf[buf_pos++] = '+';
@@ -687,6 +713,8 @@ int anum(const char *text, int len, phloat *res) {
             } else if (c == '.') {
                 /* ignore */
             } else {
+                if (!have_mant_digits)
+                    goto retry;
                 break;
             }
         } else {
@@ -701,7 +729,7 @@ int anum(const char *text, int len, phloat *res) {
             }
         }
     }
-    if (!have_mant)
+    if (!have_mant_digits)
         return false;
     if (neg_mant)
         buf[0] = '-';
@@ -728,6 +756,77 @@ int anum(const char *text, int len, phloat *res) {
     return true;
 }
 
+void fix_thousands_separators(char *buf, int *bufptr) {
+    /* First, remove the old separators... */
+    int i, j = 0;
+    char dot = flags.f.decimal_point ? '.' : ',';
+    char sep = flags.f.decimal_point ? ',' : '.';
+    int intdigits = 0;
+    int counting_intdigits = 1;
+    int nsep;
+    for (i = 0; i < *bufptr; i++) {
+        char c = buf[i];
+        if (c != sep)
+            buf[j++] = c;
+        if (c == dot || c == 24)
+            counting_intdigits = 0;
+        else if (counting_intdigits && c >= '0' && c <= '9')
+            intdigits++;
+    }
+    /* Now, put 'em back... */
+    if (!flags.f.thousands_separators) {
+        *bufptr = j;
+        return;
+    }
+    nsep = (intdigits - 1) / 3;
+    if (nsep == 0) {
+        *bufptr = j;
+        return;
+    }
+    for (i = j - 1; i >= 0; i--)
+        buf[i + nsep] = buf[i];
+    j += nsep;
+    for (i = 0; i < j; i++) {
+        char c = buf[i + nsep];
+        buf[i] = c;
+        if (nsep > 0 && c >= '0' && c <= '9') {
+            if (intdigits % 3 == 1) {
+                buf[++i] = sep;
+                nsep--;
+            }
+            intdigits--;
+        }
+    }
+    *bufptr = j;
+}
+
+void fix_base_separators(char *buf, int *bufptr) {
+    /* First, remove the old separators... */
+    int digits = 0;
+    for (int i = 0; i < *bufptr; i++) {
+        char c = buf[i];
+        if (c != ' ')
+            buf[digits++] = c;
+    }
+    /* Now, put 'em back... */
+    *bufptr = digits;
+    switch (get_base()) {
+        case  2: if (!mode_bin_sep) return; else break;
+        case  8: if (!mode_oct_sep) return; else break;
+        case 10: if (!mode_dec_sep) return; else break;
+        case 16: if (!mode_hex_sep) return; else break;
+    }
+    int nsep = (digits - 1) / 4;
+    if (nsep == 0)
+        return;
+    *bufptr += nsep;
+    for (int i = digits - 1; i >= 0; i--) {
+        buf[i + nsep] = buf[i];
+        if (((digits - i) & 3) == 0 && i != 0)
+            buf[i + (--nsep)] = ' ';
+    }
+}
+
 #if (!defined(ANDROID) && !defined(IPHONE))
 static bool always_on = false;
 bool shell_always_on(int ao) {
@@ -926,7 +1025,7 @@ void set_base(int base, bool a_thru_f) {
     flags.f.base_bit2 = (base & 4) != 0;
     flags.f.base_bit3 = (base & 8) != 0;
     if (!a_thru_f && mode_appmenu == MENU_BASE_A_THRU_F)
-        set_menu(MENULEVEL_APP, MENU_BASE);
+        set_menu(MENULEVEL_APP, MENU_BASE1);
 
     if (base != oldbase)
         print_trace();
@@ -1066,7 +1165,7 @@ void print_text(const char *text, int length, bool left_justified) {
         bitmap[i] = 0;
     for (i = 0; i < bufptr; i++) {
         int j;
-        const char *charbits = get_char(buf[i]);
+        const unsigned char *charbits = get_char(buf[i]);
         for (j = 0; j < 5; j++) {
             int x1 = i * 6 + j;
             int x2 = x1 + 1;
@@ -1179,17 +1278,8 @@ void print_wide(const char *left, int leftlen, const char *right, int rightlen)
     }
 }
 
-void print_command(int cmd, const arg_struct *arg) {
-    char buf[100];
-    int bufptr = 0;
-
-    if (cmd == CMD_NULL && !deferred_print)
-        return;
-
-    shell_annunciators(-1, -1, 1, -1, -1, -1);
-
-    if (cmd != CMD_NULL)
-        bufptr += command2buf(buf, 100, cmd, arg);
+static void print_command_2(const char *text, int len) {
+    set_annunciators(-1, -1, 1, -1, -1, -1);
 
     if (deferred_print) {
         /* If the display mode is FIX n, and the user has not entered
@@ -1225,18 +1315,59 @@ void print_command(int cmd, const arg_struct *arg) {
             }
             never_mind:;
         }
-        print_right(cmdline, cmdline_length, buf, bufptr);
+        print_right(cmdline, cmdline_length, text, len);
     } else {
         /* Normally we print commands right-justified, but if they don't fit
          * on one line, we print them left-justified, because having the excess
          * go near the right margin looks weird and confusing.
          */
-        bool left = bufptr > (flags.f.double_wide_print ? 12 : 24);
-        print_lines(buf, bufptr, left);
+        bool left = len > (flags.f.double_wide_print ? 12 : 24);
+        print_lines(text, len, left);
     }
 
     deferred_print = 0;
-    shell_annunciators(-1, -1, 0, -1, -1, -1);
+    set_annunciators(-1, -1, 0, -1, -1, -1);
+}
+
+void print_command(int cmd, const arg_struct *arg) {
+    char cbuf[100];
+    int bufptr = 0;
+    char *xbuf = NULL;
+
+    char *buf = cbuf;
+    int maxlen = 100;
+
+    if (cmd == CMD_NULL && !deferred_print)
+        return;
+
+    if (cmd != CMD_NULL) {
+        int len = 0;
+        if (cmd == CMD_EMBED || arg->type == ARGTYPE_EQN) {
+            equation_data *eqd = eq_dir->prgms[arg->val.num].eq_data;
+            if (cmd != CMD_EMBED || arg->type != ARGTYPE_NUM)
+                len = cmd_array[cmd].name_length + 1;
+            len += 2 + eqd->length;
+        } else if (arg->type == ARGTYPE_XSTR) {
+            len = 7 + arg->length;
+        }
+        if (len > maxlen) {
+            xbuf = (char *) malloc(len);
+            if (xbuf != NULL) {
+                buf = xbuf;
+                maxlen = len;
+            }
+        }
+        bufptr += command2buf(buf, maxlen, cmd, arg);
+    }
+
+    print_command_2(buf, bufptr);
+    free(xbuf);
+}
+
+void print_menu_trace(const char *name, int len) {
+    if ((flags.f.trace_print || flags.f.normal_print) && flags.f.printer_exists)
+        if (!flags.f.prgm_mode || string_equals(name, len, "PRGM", 4))
+            print_command_2(name, len);
 }
 
 void print_trace() {
@@ -1282,7 +1413,7 @@ void print_one_var(const char *name, int length) {
 int alpha_print_helper(const char *text, int length) {
     if (!flags.f.printer_exists)
         return ERR_PRINTING_IS_DISABLED;
-    shell_annunciators(-1, -1, 1, -1, -1, -1);
+    set_annunciators(-1, -1, 1, -1, -1, -1);
     if (length == 0)
         print_text(NULL, 0, true);
     else {
@@ -1302,7 +1433,7 @@ int alpha_print_helper(const char *text, int length) {
                 || (line_start > 0 && text[line_start - 1] == 10))
             print_text(text + line_start, length - line_start, true);
     }
-    shell_annunciators(-1, -1, 0, -1, -1, -1);
+    set_annunciators(-1, -1, 0, -1, -1, -1);
     return ERR_NONE;
 }
 
@@ -1512,7 +1643,7 @@ int dimension_array_ref(vartype *matrix, int4 rows, int4 columns) {
                 char *new_is_string = (char *) realloc(oldmatrix->array->is_string, size);
                 if (new_is_string != NULL)
                     oldmatrix->array->is_string = new_is_string;
-                phloat *new_data = (phloat *) realloc(oldmatrix->array->data, size * sizeof(phloat));
+                phloat *new_data = (phloat *) realloc((void *) oldmatrix->array->data, size * sizeof(phloat));
                 if (new_data != NULL)
                     oldmatrix->array->data = new_data;
                 oldmatrix->rows = rows;
@@ -1531,7 +1662,7 @@ int dimension_array_ref(vartype *matrix, int4 rows, int4 columns) {
             char *new_is_string = (char *) malloc(size);
             if (new_is_string == NULL)
                 return ERR_INSUFFICIENT_MEMORY;
-            phloat *new_data = (phloat *) realloc(oldmatrix->array->data, size * sizeof(phloat));
+            phloat *new_data = (phloat *) realloc((void *) oldmatrix->array->data, size * sizeof(phloat));
             if (new_data == NULL) {
                 free(new_is_string);
                 return ERR_INSUFFICIENT_MEMORY;
@@ -1600,7 +1731,7 @@ int dimension_array_ref(vartype *matrix, int4 rows, int4 columns) {
             oldmatrix->columns = columns;
             return ERR_NONE;
         }
-    } else /* matrix->type == TYPE_COMPLEXMATRIX */ {
+    } else if (matrix->type == TYPE_COMPLEXMATRIX) {
         vartype_complexmatrix *oldmatrix = (vartype_complexmatrix *) matrix;
         if (oldmatrix->rows == rows && oldmatrix->columns == columns)
             return ERR_NONE;
@@ -1610,7 +1741,7 @@ int dimension_array_ref(vartype *matrix, int4 rows, int4 columns) {
              */
             int4 i, oldsize;
             phloat *new_data = (phloat *)
-                    realloc(oldmatrix->array->data, 2 * size * sizeof(phloat));
+                    realloc((void *) oldmatrix->array->data, 2 * size * sizeof(phloat));
             if (new_data == NULL)
                 return ERR_INSUFFICIENT_MEMORY;
             oldsize = oldmatrix->rows * oldmatrix->columns;
@@ -1651,6 +1782,81 @@ int dimension_array_ref(vartype *matrix, int4 rows, int4 columns) {
             oldmatrix->columns = columns;
             return ERR_NONE;
         }
+    } else /* matrix->type == TYPE_LIST */ {
+        if (columns != 1)
+            return ERR_DIMENSION_ERROR;
+        vartype_list *oldlist = (vartype_list *) matrix;
+        if (oldlist->size == size)
+            return ERR_NONE;
+        if (oldlist->array->refcount == 1) {
+            /* Since there are no shared references to this array,
+             * I can modify it in place using a realloc().
+             */
+            if (oldlist->size > size) {
+                for (int4 i = size; i < oldlist->size; i++) {
+                    free_vartype(oldlist->array->data[i]);
+                    oldlist->array->data[i] = NULL;
+                }
+                vartype **new_data = (vartype **) realloc(oldlist->array->data, size * sizeof(vartype *));
+                /* Note: If the realloc() fails to shrink the array, we just keep
+                 * using the existing one, basically pretending that it succeeded.
+                 */
+                if (new_data != NULL)
+                    oldlist->array->data = new_data;
+                oldlist->size = size;
+                return ERR_NONE;
+            } else {
+                vartype **new_data = (vartype **) realloc(oldlist->array->data, size * sizeof(vartype *));
+                if (new_data == NULL)
+                    return ERR_INSUFFICIENT_MEMORY;
+                for (int4 i = oldlist->size; i < size; i++) {
+                    new_data[i] = new_real(0);
+                    if (new_data[i] == NULL) {
+                        /* Argh. Roll back everything and give up. */
+                        for (int4 j = oldlist->size; j < i; j++) {
+                            free_vartype(new_data[j]);
+                            new_data[j] = NULL;
+                        }
+                        vartype **reverted_data = (vartype **) realloc(new_data, oldlist->size * sizeof(vartype *));
+                        oldlist->array->data = reverted_data == NULL ? new_data : reverted_data;
+                        return ERR_INSUFFICIENT_MEMORY;
+                    }
+                }
+                oldlist->array->data = new_data;
+                oldlist->size = size;
+                return ERR_NONE;
+            }
+        } else {
+            /* There are shared references to the list. This means I
+             * can't realloc() it; I'm going to allocate a brand-new instance,
+             * and copy the contents from the old instance. I don't use
+             * disentangle(); that's only useful if you want to eliminate
+             * shared references without resizing.
+             */
+            list_data *new_array = (list_data *) malloc(sizeof(list_data));
+            if (new_array == NULL)
+                return ERR_INSUFFICIENT_MEMORY;
+            new_array->data = (vartype **) malloc(size * sizeof(vartype *));
+            if (new_array->data == NULL) {
+                free(new_array);
+                return ERR_INSUFFICIENT_MEMORY;
+            }
+            for (int4 i = 0; i < size; i++) {
+                new_array->data[i] = i < oldlist->size ? dup_vartype(oldlist->array->data[i]) : new_real(0);
+                if (new_array->data[i] == NULL) {
+                    for (int4 j = 0; j < i; j++)
+                        free_vartype(new_array->data[j]);
+                    free(new_array->data);
+                    free(new_array);
+                    return ERR_INSUFFICIENT_MEMORY;
+                }
+            }
+            new_array->refcount = 1;
+            oldlist->array->refcount--;
+            oldlist->array = new_array;
+            oldlist->size = size;
+            return ERR_NONE;
+        }
     }
 }
 
@@ -2103,7 +2309,21 @@ int easy_phloat2string(phloat d, char *buf, int buflen, int base_mode) {
 }
 
 int real2buf(char *buf, phloat x, const char *format, bool force_decimal) {
+    bool saved_binsep, saved_octsep, saved_decsep, saved_hexsep;
+    if (!force_decimal) {
+        saved_binsep = mode_bin_sep;
+        saved_octsep = mode_oct_sep;
+        saved_decsep = mode_dec_sep;
+        saved_hexsep = mode_hex_sep;
+        mode_bin_sep = mode_oct_sep = mode_dec_sep = mode_hex_sep = false;
+    }
     int bufptr = phloat2string(x, buf, 49, force_decimal ? 0 : 1, 0, 3, 0, MAX_MANT_DIGITS, format);
+    if (!force_decimal) {
+        mode_bin_sep = saved_binsep;
+        mode_oct_sep = saved_octsep;
+        mode_dec_sep = saved_decsep;
+        mode_hex_sep = saved_hexsep;
+    }
     /* Convert small-caps 'E' to regular 'e' */
     for (int i = 0; i < bufptr; i++)
         if (buf[i] == 24)
@@ -2145,18 +2365,21 @@ vartype_list *get_path() {
     return NULL;
 }
 
-vartype *matedit_get() {
-    if (matedit_mode != 1 && matedit_mode != 3)
-        return NULL;
+int matedit_get(vartype **res) {
+    if (matedit_mode == 0)
+        return ERR_NONEXISTENT;
+
     vartype *m = NULL;
-    if (matedit_dir <= 0) {
+    if (matedit_mode == 2) {
+        m = matedit_x;
+    } else if (matedit_dir <= 0) {
         int level = -matedit_dir;
         for (int i = local_vars_count - 1; i >= 0; i--) {
             var_struct *lv = local_vars + i;
             if ((lv->flags & VAR_PRIVATE) != 0)
                 continue;
             if (lv->level < level)
-                return NULL;
+                return ERR_NONEXISTENT;
             if (lv->level == level && string_equals(matedit_name, matedit_length, lv->name, lv->length)) {
                 m = lv->value;
                 break;
@@ -2165,7 +2388,7 @@ vartype *matedit_get() {
     } else {
         directory *dir = get_dir(matedit_dir);
         if (dir == NULL)
-            return NULL;
+            return ERR_NONEXISTENT;
         for (int i = dir->vars_count - 1; i >= 0; i--) {
             var_struct *gv = dir->vars + i;
             if (string_equals(matedit_name, matedit_length, gv->name, gv->length)) {
@@ -2174,14 +2397,35 @@ vartype *matedit_get() {
             }
         }
     }
-    // Apart from the check for m == NULL, the following checks *should* be unnecessary,
-    // we're already preventing those scenarios. Or that's what we're trying, anyway...
+    int err = ERR_NONEXISTENT;
     if (m == NULL) {
         bad_matrix:
-        if (matedit_mode == 3)
+        if (matedit_mode == 2 || matedit_mode == 3)
             leave_matrix_editor();
-        return NULL;
-    } else if (m->type == TYPE_REALMATRIX) {
+        return err;
+    }
+
+    for (int i = 0; i < matedit_stack_depth; i++) {
+        if (m->type != TYPE_LIST) {
+            err = i == 0 ? ERR_INVALID_TYPE : ERR_INVALID_DATA;
+            goto bad_matrix;
+        }
+        vartype_list *list = (vartype_list *) m;
+        if (matedit_stack[i].coord >= list->size) {
+            err = ERR_INVALID_DATA;
+            goto bad_matrix;
+        }
+        m = list->array->data[matedit_stack[i].coord];
+    }
+
+    if (m->type != TYPE_REALMATRIX && m->type != TYPE_COMPLEXMATRIX && m->type != TYPE_LIST) {
+        err = matedit_stack_depth == 0 ? ERR_INVALID_TYPE : ERR_INVALID_DATA;
+        goto bad_matrix;
+    }
+
+    // The following checks *should* be unnecessary, we're already preventing
+    // those scenarios. Or that's what we're trying, anyway...
+    if (m->type == TYPE_REALMATRIX) {
         vartype_realmatrix *rm = (vartype_realmatrix *) m;
         if (matedit_i >= rm->rows || matedit_j >= rm->columns)
             matedit_i = matedit_j = 0;
@@ -2189,13 +2433,32 @@ vartype *matedit_get() {
         vartype_complexmatrix *cm = (vartype_complexmatrix *) m;
         if (matedit_i >= cm->rows || matedit_j >= cm->columns)
             matedit_i = matedit_j = 0;
-    } else
-        goto bad_matrix;
-    return m;
+    } else { // m->type == TYPE_LIST
+        vartype_list *list = (vartype_list *) m;
+        if (matedit_i >= list->size)
+            matedit_i = 0;
+        matedit_j = 0;
+    }
+
+    *res = m;
+    return ERR_NONE;
 }
 
 void leave_matrix_editor() {
     set_appmenu_exitcallback(0);
     set_menu(MENULEVEL_APP, MENU_NONE);
     matedit_mode = 0;
+    free(matedit_stack);
+    matedit_stack = NULL;
+    matedit_stack_depth = 0;
+}
+
+void switch_30_and_94(char *text, int length) {
+    for (int i = 0; i < length; i++) {
+        char c = text[i];
+        if (c == 30)
+            text[i] = 94;
+        else if (c == 94)
+            text[i] = 30;
+    }
 }
