@@ -282,8 +282,8 @@ static int div_cc_completion2(int error, vartype_complexmatrix *a, int4 *perm,
     return linalg_div_completion(error, linalg_div_result);
 }
 
-static int small_inv_r(vartype_realmatrix *ma, void (*completion)(vartype *));
-static int small_inv_c(vartype_complexmatrix *ma, void (*completion)(vartype *));
+static int small_inv_r(vartype_realmatrix *ma, int (*completion)(int, vartype *));
+static int small_inv_c(vartype_complexmatrix *ma, int (*completion)(int, vartype *));
 static int matrix_mul_rr(vartype_realmatrix *left, vartype_realmatrix *right, int (*completion)(int, vartype *));
 static int matrix_mul_cr(vartype_complexmatrix *left, vartype_realmatrix *right, int (*completion)(int, vartype *));
 static int matrix_mul_rc(vartype_realmatrix *left, vartype_complexmatrix *right, int (*completion)(int, vartype *));
@@ -292,8 +292,9 @@ static int matrix_mul_cc(vartype_complexmatrix *left, vartype_complexmatrix *rig
 static vartype *small_div_res;
 static int (*small_div_completion)(int, vartype *);
 
-static void small_div_completion_1(vartype *v) {
+static int small_div_completion_1(int error, vartype *v) {
     small_div_res = v;
+    return error;
 }
 
 static int small_div_completion_2(int err, vartype *v) {
@@ -1004,7 +1005,7 @@ int linalg_mul(const vartype *left, const vartype *right,
 /***** Matrix inverse *****/
 /**************************/
 
-static void (*linalg_inv_completion)(vartype *det);
+static int (*linalg_inv_completion)(int error, vartype *det);
 static vartype *linalg_inv_result;
 
 static int inv_r_completion1(int error, vartype_realmatrix *a, int4 *perm,
@@ -1016,7 +1017,7 @@ static int inv_c_completion1(int error, vartype_complexmatrix *a, int4 *perm,
 static int inv_c_completion2(int error, vartype_complexmatrix *a, int4 *perm,
                                 vartype_complexmatrix *b);
 
-int linalg_inv(const vartype *src, void (*completion)(vartype *)) {
+int linalg_inv(const vartype *src, int (*completion)(int, vartype *)) {
     int4 n;
     int4 *perm;
     if (src->type == TYPE_REALMATRIX) {
@@ -1024,24 +1025,24 @@ int linalg_inv(const vartype *src, void (*completion)(vartype *)) {
         vartype *lu, *inv;
         n = ma->rows;
         if (n != ma->columns)
-            return ERR_DIMENSION_ERROR;
+            return completion(ERR_DIMENSION_ERROR, NULL);
         if (contains_strings(ma))
-            return ERR_ALPHA_DATA_IS_INVALID;
+            return completion(ERR_ALPHA_DATA_IS_INVALID, NULL);
         if (n <= 2)
             return small_inv_r(ma, completion);
         lu = new_realmatrix(n, n);
         if (lu == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         inv = new_realmatrix(n, n);
         if (inv == NULL) {
             free_vartype(lu);
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         }
         perm = (int4 *) malloc(n * sizeof(int4));
         if (perm == NULL) {
             free_vartype(lu);
             free_vartype(inv);
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         }
         matrix_copy(lu, src);
         linalg_inv_completion = completion;
@@ -1052,22 +1053,22 @@ int linalg_inv(const vartype *src, void (*completion)(vartype *)) {
         vartype *lu, *inv;
         n = ma->rows;
         if (n != ma->columns)
-            return ERR_DIMENSION_ERROR;
+            return completion(ERR_DIMENSION_ERROR, NULL);
         if (n <= 2)
             return small_inv_c(ma, completion);
         lu = new_complexmatrix(n, n);
         if (lu == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         inv = new_complexmatrix(n, n);
         if (inv == NULL) {
             free_vartype(lu);
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         }
         perm = (int4 *) malloc(n * sizeof(int4));
         if (perm == NULL) {
             free_vartype(lu);
             free_vartype(inv);
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         }
         matrix_copy(lu, src);
         linalg_inv_completion = completion;
@@ -1083,7 +1084,7 @@ static int inv_r_completion1(int error, vartype_realmatrix *a, int4 *perm,
         free_vartype(linalg_inv_result);
         free_vartype((vartype *) a);
         free(perm);
-        return error;
+        return linalg_inv_completion(error, NULL);
     } else {
         int4 i, n = a->rows;
         vartype_realmatrix *inv = (vartype_realmatrix *) linalg_inv_result;
@@ -1097,11 +1098,9 @@ static int inv_r_completion2(int error, vartype_realmatrix *a, int4 *perm,
                                 vartype_realmatrix *b) {
     if (error != ERR_NONE)
         free_vartype(linalg_inv_result); /* Note: linalg_inv_result == b */
-    else
-        linalg_inv_completion(linalg_inv_result);
     free_vartype((vartype *) a);
     free(perm);
-    return error;
+    return linalg_inv_completion(error, linalg_inv_result);
 }
 
 static int inv_c_completion1(int error, vartype_complexmatrix *a, int4 *perm,
@@ -1110,7 +1109,7 @@ static int inv_c_completion1(int error, vartype_complexmatrix *a, int4 *perm,
         free_vartype(linalg_inv_result);
         free_vartype((vartype *) a);
         free(perm);
-        return error;
+        return linalg_inv_completion(error, NULL);
     } else {
         int4 i, n = a->rows;
         vartype_complexmatrix *inv =
@@ -1125,11 +1124,9 @@ static int inv_c_completion2(int error, vartype_complexmatrix *a, int4 *perm,
                                 vartype_complexmatrix *b) {
     if (error != ERR_NONE)
         free_vartype(linalg_inv_result); /* Note: linalg_inv_result == b */
-    else
-        linalg_inv_completion(linalg_inv_result);
     free_vartype((vartype *) a);
     free(perm);
-    return error;
+    return linalg_inv_completion(error, linalg_inv_result);
 }
 
 static int small_det_r(vartype_realmatrix *m, phloat *r);
@@ -1141,69 +1138,66 @@ static int small_inv_completion(int err, vartype *res) {
     return err;
 }
 
-static int small_inv_r(vartype_realmatrix *ma, void (*completion)(vartype *)) {
+static int small_inv_r(vartype_realmatrix *ma, int (*completion)(int, vartype *)) {
     if (ma->rows == 1) {
         phloat x = ma->array->data[0];
         if (x == 0) {
             if (core_settings.matrix_singularmatrix)
-                return ERR_SINGULAR_MATRIX;
+                return completion(ERR_SINGULAR_MATRIX, NULL);
             x = POS_HUGE_PHLOAT;
         } else {
             x = phloat(1) / x;
             int inf = p_isinf(x);
             if (inf != 0) {
                 if (core_settings.matrix_outofrange && !flags.f.range_error_ignore)
-                    return ERR_OUT_OF_RANGE;
+                    return completion(ERR_OUT_OF_RANGE, NULL);
                 x = inf > 0 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
             }
         }
         vartype *r = new_realmatrix(1, 1);
         if (r == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         ((vartype_realmatrix *) r)->array->data[0] = x;
-        completion(r);
-        return ERR_NONE;
+        return completion(ERR_NONE, r);
     }
     phloat det;
     int err = small_det_r(ma, &det);
     if (det == 0) {
         if (core_settings.matrix_singularmatrix)
-            return ERR_SINGULAR_MATRIX;
+            return completion(ERR_SINGULAR_MATRIX, NULL);
         vartype_realmatrix *sm = (vartype_realmatrix *) new_realmatrix(2, 2);
         if (sm == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         sm->array->data[0] = POS_HUGE_PHLOAT;
         sm->array->data[3] = POS_HUGE_PHLOAT;
-        completion((vartype *) sm);
-        return ERR_NONE;
+        return completion(ERR_NONE, (vartype *) sm);
     }
     vartype *d = new_real(det);
     if (d == NULL)
-        return ERR_INSUFFICIENT_MEMORY;
+        return completion(ERR_INSUFFICIENT_MEMORY, NULL);
     bool saved_range_error_ignore = flags.f.range_error_ignore;
     flags.f.range_error_ignore = !core_settings.matrix_outofrange || flags.f.range_error_ignore;
     err = generic_div(d, (vartype *) ma, small_inv_completion);
     flags.f.range_error_ignore = saved_range_error_ignore;
     free_vartype(d);
     if (err != ERR_NONE)
-        return err;
+        return completion(err, NULL);
     phloat *data = ((vartype_realmatrix *) small_inv_res)->array->data;
     phloat t = data[0];
     data[0] = data[3];
     data[3] = t;
     data[1] = -data[1];
     data[2] = -data[2];
-    completion(small_inv_res);
-    return ERR_NONE;
+    return completion(ERR_NONE, small_inv_res);
 }
 
-static int small_inv_c(vartype_complexmatrix *ma, void (*completion)(vartype *)) {
+static int small_inv_c(vartype_complexmatrix *ma, int (*completion)(int, vartype *)) {
     if (ma->rows == 1) {
         phloat xre = ma->array->data[0];
         phloat xim = ma->array->data[1];
         if (xre == 0 && xim == 0) {
             if (core_settings.matrix_singularmatrix)
-                return ERR_SINGULAR_MATRIX;
+                return completion(ERR_SINGULAR_MATRIX, NULL);
             xre = POS_HUGE_PHLOAT;
             xim = 0;
         } else {
@@ -1212,39 +1206,37 @@ static int small_inv_c(vartype_complexmatrix *ma, void (*completion)(vartype *))
             int err = math_inv(xre, xim, &xre, &xim);
             flags.f.range_error_ignore = saved_range_error_ignore;
             if (err != ERR_NONE)
-                return err;
+                return completion(err, NULL);
         }
         vartype *r = new_complexmatrix(1, 1);
         if (r == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         ((vartype_complexmatrix *) r)->array->data[0] = xre;
         ((vartype_complexmatrix *) r)->array->data[1] = xim;
-        completion(r);
-        return ERR_NONE;
+        return completion(ERR_NONE, r);
     }
     phloat dre, dim;
     int err = small_det_c(ma, &dre, &dim);
     if (dre == 0 && dim == 0) {
         if (core_settings.matrix_singularmatrix)
-            return ERR_SINGULAR_MATRIX;
+            return completion(ERR_SINGULAR_MATRIX, NULL);
         vartype_complexmatrix *sm = (vartype_complexmatrix *) new_complexmatrix(2, 2);
         if (sm == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         sm->array->data[0] = POS_HUGE_PHLOAT;
         sm->array->data[6] = POS_HUGE_PHLOAT;
-        completion((vartype *) sm);
-        return ERR_NONE;
+        return completion(ERR_NONE, (vartype *) sm);
     }
     vartype *d = new_complex(dre, dim);
     if (d == NULL)
-        return ERR_INSUFFICIENT_MEMORY;
+        return completion(ERR_INSUFFICIENT_MEMORY, NULL);
     bool saved_range_error_ignore = flags.f.range_error_ignore;
     flags.f.range_error_ignore = !core_settings.matrix_outofrange || flags.f.range_error_ignore;
     err = generic_div(d, (vartype *) ma, small_inv_completion);
     flags.f.range_error_ignore = saved_range_error_ignore;
     free_vartype(d);
     if (err != ERR_NONE)
-        return err;
+        return completion(err, NULL);
     phloat *data = ((vartype_complexmatrix *) small_inv_res)->array->data;
     phloat t = data[0];
     data[0] = data[6];
@@ -1256,8 +1248,7 @@ static int small_inv_c(vartype_complexmatrix *ma, void (*completion)(vartype *))
     data[3] = -data[3];
     data[4] = -data[4];
     data[5] = -data[5];
-    completion(small_inv_res);
-    return ERR_NONE;
+    return completion(ERR_NONE, small_inv_res);
 }
 
 
@@ -1265,7 +1256,7 @@ static int small_inv_c(vartype_complexmatrix *ma, void (*completion)(vartype *))
 /***** Matrix determinant *****/
 /******************************/
 
-static void (*linalg_det_completion)(vartype *det);
+static int (*linalg_det_completion)(int, vartype *det);
 static bool linalg_det_prev_sm_err;
 
 static int det_r_completion(int error, vartype_realmatrix *a, int4 *perm,
@@ -1273,38 +1264,37 @@ static int det_r_completion(int error, vartype_realmatrix *a, int4 *perm,
 static int det_c_completion(int error, vartype_complexmatrix *a, int4 *perm,
                                     phloat det_re, phloat det_im);
 
-int linalg_det(const vartype *src, void (*completion)(vartype *)) {
+int linalg_det(const vartype *src, int (*completion)(int, vartype *)) {
     int4 n;
     int4 *perm;
     if (src->type == TYPE_REALMATRIX) {
         vartype_realmatrix *ma = (vartype_realmatrix *) src;
         n = ma->rows;
         if (n != ma->columns)
-            return ERR_DIMENSION_ERROR;
+            return completion(ERR_DIMENSION_ERROR, NULL);
         if (contains_strings(ma))
-            return ERR_ALPHA_DATA_IS_INVALID;
+            return completion(ERR_ALPHA_DATA_IS_INVALID, NULL);
         if (n <= 2) {
             phloat d;
             int err = small_det_r(ma, &d);
             if (err != ERR_NONE)
-                return err;
+                return completion(err, NULL);
             vartype *v = new_real(d);
             if (v == NULL)
-                return ERR_INSUFFICIENT_MEMORY;
-            completion(v);
-            return ERR_NONE;
+                return completion(ERR_INSUFFICIENT_MEMORY, NULL);
+            return completion(ERR_NONE, v);
         }
         ma = (vartype_realmatrix *) dup_vartype(src);
         if (ma == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         if (!disentangle((vartype *) ma)) {
             free_vartype((vartype *) ma);
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         }
         perm = (int4 *) malloc(n * sizeof(int4));
         if (perm == NULL) {
             free_vartype((vartype *) ma);
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         }
 
         /* Before calling lu_decomp_r, make sure the 'singular matrix'
@@ -1323,30 +1313,29 @@ int linalg_det(const vartype *src, void (*completion)(vartype *)) {
         vartype_complexmatrix *ma = (vartype_complexmatrix *) src;
         n = ma->rows;
         if (n != ma->columns)
-            return ERR_DIMENSION_ERROR;
+            return completion(ERR_DIMENSION_ERROR, NULL);
         if (n <= 2) {
             phloat dre, dim;
             int err = small_det_c(ma, &dre, &dim);
             if (err != ERR_NONE)
-                return err;
+                return completion(err, NULL);
             vartype *v = new_complex(dre, dim);
             if (v == NULL)
-                return ERR_INSUFFICIENT_MEMORY;
-            completion(v);
-            return ERR_NONE;
+                return completion(ERR_INSUFFICIENT_MEMORY, NULL);
+            return completion(ERR_NONE, v);
         }
         ma = (vartype_complexmatrix *) dup_vartype(src);
         if (ma == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         if (!disentangle((vartype *) ma)) {
             free_vartype((vartype *) ma);
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         }
         n = ma->rows;
         perm = (int4 *) malloc(n * sizeof(int4));
         if (perm == NULL) {
             free_vartype((vartype *) ma);
-            return ERR_INSUFFICIENT_MEMORY;
+            return completion(ERR_INSUFFICIENT_MEMORY, NULL);
         }
 
         /* Before calling lu_decomp_c, make sure the 'singular matrix'
@@ -1391,9 +1380,7 @@ static int det_r_completion(int error, vartype_realmatrix *a, int4 *perm,
             error = ERR_INSUFFICIENT_MEMORY;
     }
 
-    if (error == ERR_NONE)
-        linalg_det_completion(det_v);
-    return error;
+    return linalg_det_completion(error, det_v);
 }
 
 static int det_c_completion(int error, vartype_complexmatrix *a, int4 *perm,
@@ -1430,9 +1417,7 @@ static int det_c_completion(int error, vartype_complexmatrix *a, int4 *perm,
             error = ERR_INSUFFICIENT_MEMORY;
     }
 
-    if (error == ERR_NONE)
-        linalg_det_completion(det_v);
-    return error;
+    return linalg_det_completion(error, det_v);
 }
 
 static phloat dot_2d(phloat a1, phloat a2, phloat a3, phloat a4, bool add, int *scale = NULL) {
