@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -35,6 +35,10 @@ bool pgm_index::is_editable() {
     return dir == cwd->id;
 }
 
+bool pgm_index::is_locked() {
+    return dir_list[dir]->prgms[idx].locked;
+}
+
 // We cache vartype_real, vartype_complex, and vartype_string instances, to
 // cut down on the malloc/free overhead.
 
@@ -194,22 +198,34 @@ vartype *new_list(int4 size) {
     return (vartype *) list;
 }
 
-vartype *new_equation(const char *text, int4 len, bool compat_mode, int *errpos) {
-    int eqn_index = new_eqn_idx();
-    if (eqn_index == -1)
-        return NULL;
+equation_data *new_equation_data(const char *text, int4 length, bool compat_mode, int *errpos, int eqn_index) {
+    *errpos = -1;
+    if (eqn_index == -1) {
+        for (int i = 0; i < eq_dir->prgms_count; i++) {
+            equation_data *eqd = eq_dir->prgms[i].eq_data;
+            if (eqd == NULL)
+                continue;
+            if (!eqd->compatModeEmbedded && eqd->compatMode != compat_mode)
+                continue;
+            if (!string_equals(eqd->text, eqd->length, text, length))
+                continue;
+            return eqd;
+        }
+        eqn_index = new_eqn_idx();
+        if (eqn_index == -1)
+            return NULL;
+    }
     equation_data *eqd = new (std::nothrow) equation_data;
     if (eqd == NULL)
         return NULL;
-    *errpos = -1;
-    eqd->length = len;
-    eqd->text = (char *) malloc(len);
-    if (eqd->text == NULL && len != 0) {
+    eqd->length = length;
+    eqd->text = (char *) malloc(length);
+    if (eqd->text == NULL && length != 0) {
         delete eqd;
         return NULL;
     }
-    memcpy(eqd->text, text, len);
-    eqd->ev = Parser::parse(std::string(text, len), &compat_mode, &eqd->compatModeEmbedded, errpos);
+    memcpy(eqd->text, text, length);
+    eqd->ev = Parser::parse(std::string(text, length), &compat_mode, &eqd->compatModeEmbedded, errpos);
     if (eqd->ev == NULL) {
         delete eqd;
         return NULL;
@@ -217,14 +233,6 @@ vartype *new_equation(const char *text, int4 len, bool compat_mode, int *errpos)
     eqd->eqn_index = eqn_index;
     eqd->compatMode = compat_mode;
 
-    vartype_equation *eq = (vartype_equation *) malloc(sizeof(vartype_equation));
-    if (eq == NULL) {
-        delete eqd;
-        return NULL;
-    }
-    eq->type = TYPE_EQUATION;
-    eq->data = eqd;
-    eqd->refcount = 1;
     CodeMap *map = new (std::nothrow) CodeMap;
     eq_dir->prgms[eqn_index].eq_data = eqd;
     Parser::generateCode(eqd->ev, eq_dir->prgms + eqn_index, map);
@@ -235,11 +243,29 @@ vartype *new_equation(const char *text, int4 len, bool compat_mode, int *errpos)
     eqd->map = map;
     if (eq_dir->prgms[eqn_index].text == NULL) {
         // Code generator failure
-        free_vartype((vartype *) eq);
+        eq_dir->prgms[eqn_index].eq_data = NULL;
+        delete eqd;
         return NULL;
     }
 
-    return (vartype *) eq;
+    return eqd;
+}
+
+vartype *new_equation(const char *text, int4 len, bool compat_mode, int *errpos) {
+    *errpos = -1;
+    vartype_equation *eq = (vartype_equation *) malloc(sizeof(vartype_equation));
+    if (eq == NULL)
+        return NULL;
+    equation_data *eqd = new_equation_data(text, len, compat_mode, errpos, -1);
+    if (eqd == NULL) {
+        free(eq);
+        return NULL;
+    } else {
+        eq->type = TYPE_EQUATION;
+        eq->data = eqd;
+        eqd->refcount++;
+        return (vartype *) eq;
+    }
 }
 
 vartype *new_equation(equation_data *eqd) {
@@ -272,8 +298,8 @@ vartype *new_unit(phloat value, const char *text, int4 length) {
             c = '/';
         else if (c == '\1')
             c = '*';
-        else if (c == '\36')
-            c = '^';
+        else if (c == '^')
+            c = '\36';
         u->text[i] = c;
     }
     u->length = length;
@@ -375,21 +401,7 @@ void free_vartype(vartype *v) {
         }
         case TYPE_EQUATION: {
             vartype_equation *eq = (vartype_equation *) v;
-            if (--(eq->data->refcount) == 0) {
-                int eqn_index = eq->data->eqn_index;
-                if (current_prgm.dir == eq_dir->id && current_prgm.idx == eqn_index) {
-                    // This shouldn't be possible while a program is running,
-                    // but could happen if the user initiates an action that
-                    // clears the RTN stack, while execution is halted in
-                    // generated code.
-                    current_prgm.set(cwd->id, cwd->prgms_count - 1);
-                    pc = cwd->prgms[cwd->prgms_count - 1].size - 2;
-                }
-                free(eq_dir->prgms[eqn_index].text);
-                eq_dir->prgms[eqn_index].text = NULL;
-                eq_dir->prgms[eqn_index].eq_data = NULL;
-                delete eq->data;
-            }
+            remove_equation_reference(eq->data->eqn_index);
             free(eq);
             break;
         }
@@ -1044,9 +1056,12 @@ static bool var_exists(int section, int type) {
             return section == CATSECT_CPX;
         case TYPE_REALMATRIX:
         case TYPE_COMPLEXMATRIX:
-            return section == CATSECT_MAT || section == CATSECT_MAT_ONLY;
+            return section == CATSECT_MAT || section == CATSECT_MAT_ONLY
+                || section == CATSECT_MAT_LIST || section == CATSECT_MAT_LIST_ONLY;
         case TYPE_LIST:
-            return section == CATSECT_LIST || section == CATSECT_LIST_STR_ONLY;
+            return section == CATSECT_LIST || section == CATSECT_LIST_ONLY
+                || section == CATSECT_LIST_STR_ONLY
+                || section == CATSECT_MAT_LIST || section == CATSECT_MAT_LIST_ONLY;
         case TYPE_EQUATION:
             return section == CATSECT_EQN || section == CATSECT_EQN_ONLY;
         case TYPE_UNIT:
@@ -1133,7 +1148,7 @@ int matrix_copy(vartype *dst, const vartype *src) {
             int4 size = s->rows * s->columns;
             free_long_strings(d->array->is_string, d->array->data, size);
             memset(d->array->is_string, 0, size);
-            memcpy(d->array->data, s->array->data, size * sizeof(phloat));
+            memcpy((void *) d->array->data, (const void *) s->array->data, size * sizeof(phloat));
             return ERR_NONE;
         } else if (dst->type == TYPE_COMPLEXMATRIX) {
             vartype_complexmatrix *d = (vartype_complexmatrix *) dst;
@@ -1156,20 +1171,20 @@ int matrix_copy(vartype *dst, const vartype *src) {
         if (s->rows != d->rows || s->columns != d->columns)
             return ERR_DIMENSION_ERROR;
         int4 size = s->rows * s->columns * 2;
-        memcpy(d->array->data, s->array->data, size * sizeof(phloat));
+        memcpy((void *) d->array->data, (const void *) s->array->data, size * sizeof(phloat));
         return ERR_NONE;
     } else
         return ERR_INVALID_TYPE;
 }
 
-static vloc lookup_private_var(const char *name, int namelength) {
+static vloc lookup_private_var(const char *name, int namelength, bool allow_calling_frames) {
     int level = get_rtn_level();
     int i, j;
     for (i = local_vars_count - 1; i >= 0; i--) {
         int vlevel = local_vars[i].level;
         if (vlevel == -1)
             continue;
-        if (vlevel < level)
+        if (!allow_calling_frames && vlevel < level)
             break;
         if ((local_vars[i].flags & VAR_PRIVATE) == 0)
             continue;
@@ -1184,12 +1199,12 @@ static vloc lookup_private_var(const char *name, int namelength) {
     return vloc();
 }
 
-vartype *recall_private_var(const char *name, int namelength) {
-    return lookup_private_var(name, namelength).value();
+vartype *recall_private_var(const char *name, int namelength, bool allow_calling_frames) {
+    return lookup_private_var(name, namelength, allow_calling_frames).value();
 }
 
 vartype *recall_and_purge_private_var(const char *name, int namelength) {
-    vloc varindex = lookup_private_var(name, namelength);
+    vloc varindex = lookup_private_var(name, namelength, false);
     if (varindex.not_found())
         return NULL;
     vartype *ret = varindex.value();
@@ -1200,7 +1215,7 @@ vartype *recall_and_purge_private_var(const char *name, int namelength) {
 }
 
 int store_private_var(const char *name, int namelength, vartype *value) {
-    vloc varindex = lookup_private_var(name, namelength);
+    vloc varindex = lookup_private_var(name, namelength, false);
     int i;
     if (varindex.not_found()) {
         if (local_vars_count == local_vars_capacity) {
@@ -1234,3 +1249,58 @@ int store_stack_reference(vartype *v) {
         free_vartype(v2);
     return err;
 }
+
+void reparse_all_equations() {
+    // NOTE: We can safely assume that re-parsing all equations will succeed,
+    // because we re-parse all equations anyway when the state file is loaded,
+    // in order to re-create their parse trees. (The generated code and the
+    // code map are loaded from the state file.) This means that parser changes
+    // that cause formerly valid equations to become invalid will be caught by
+    // unpersist_vartype(), and those equations will be converted to strings,
+    // before we even get here. The re-parsing we're doing here is in order to
+    // re-generate the code, in cases when code generator bugs have been fixed
+    // or the semantics of generated code have changed.
+    for (int4 i = 0; i < eq_dir->prgms_capacity; i++) {
+        prgm_struct *prgm = eq_dir->prgms + i;
+        if (prgm->text == NULL)
+            continue;
+        prgm_struct old_prgm = *prgm;
+        equation_data *old_eqd = old_prgm.eq_data;
+        prgm->size = prgm->capacity = 0;
+        prgm->text = NULL;
+        prgm->eq_data = NULL;
+        int errpos = -1;
+        equation_data *new_eqd = new_equation_data(old_eqd->text, old_eqd->length, flags.f.eqn_compat, &errpos, i);
+        if (new_eqd == NULL) {
+            // Should never happen, but now that we're here, let's at least
+            // try to handle it gracefully, by keeping the old equation.
+            *prgm = old_prgm;
+        } else {
+            // I can't just replace the old equation_data object, because of
+            // all the vartype_equation objects referencing it. Hence, we
+            // copy the parse tree and code map from the new equation_data
+            // into the old one, and then delete the new one.
+            delete old_eqd->ev;
+            delete old_eqd->map;
+            old_eqd->ev = new_eqd->ev;
+            old_eqd->map = new_eqd->map;
+            new_eqd->ev = NULL;
+            new_eqd->map = NULL;
+            delete new_eqd;
+            free(old_prgm.text);
+            prgm->eq_data = old_prgm.eq_data;
+        }
+    }
+}
+
+void remove_equation_reference(int4 id) {
+    equation_data *eqd = eq_dir->prgms[id].eq_data;
+    if (--(eqd->refcount) > 0)
+        return;
+    equation_deleted(id);
+    count_embed_references(eq_dir, id, false);
+    free(eq_dir->prgms[id].text);
+    eq_dir->prgms[id].text = NULL;
+    eq_dir->prgms[id].eq_data = NULL;
+    delete eqd;
+}
