@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -26,6 +26,6 @@
  * set to the number of extra pixels needed, and dup_first_y and dup_last_y
  * specify the strip that has to be tiled to fill up the extra pixels.
  */
-int shell_loadimage(int extra, int dup_first_y, int dup_last_y);
+bool shell_loadimage(int extra, int dup_first_y, int dup_last_y);
 
 #endif
